% H2Controller
%% Linearized physical model

T_delta = diag([1,1,1])*foil.time_const;
R_star = eye(3);
R_U_0 = zeros(3);
R_U_0(1,3) = -U_0;


% Mass matrix
S_rg = Smtrx(r_g);

M_tot    = [boat.mass*eye(3), -boat.mass*S_rg
        boat.mass*S_rg, I_b];
    
% Force-attack points
x_foil = force_attack_point.foil.r_FR(1);
y_foil = force_attack_point.foil.r_FR(2);
z_foil = force_attack_point.foil.r_FR(3);

% B_delta matrix
AR_F = 0.5*foil.span/foil.cord_length;
C_D_F_delta = 0*2*foil.C_L_0*foil.C_L_alpha/(pi*1*AR_F);

B_delta = [0,0,-foil.C_D_alpha
    0,0,0
    0,0, -foil.C_L_alpha
    foil.C_L_alpha*y_foil, 0,0
    0, foil.C_L_alpha*x_foil, -C_D_F_delta*z_foil
    foil.C_D_alpha*y_foil,0,0]*water_density*foil.S*U_0^2;

% Gravity
G_star = boat.mass*g*[0,0,1;0,-1,0;0,0,0; 0,r_g(3),0;0,0,r_g(3);0,-r_g(1),-r_g(2)];
Lg = zeros(6,3);
Lg(1,3) = 1;
Lg(2,2) = -1;

% Selection matrix
L_3DOF = zeros(3,6);
L_3DOF(1,3) = 1;
L_3DOF(2,4) = 1;
L_3DOF(3,5) = 1;

% Coriolis
% C_star = boat.mass*U_0*[zeros(6,2),[-1;-r_g(2);r_g(1)]];
Lc = zeros(6,3);
Lc(3,3) = -1;

% Damping
D_star = sumLinearizedDampingMatrices_liftingFoils_2(foil,force_attack_point.foil)*0.25*water_density*foil.S*U_0;

% SS model
clear G;
G.A = [-inv(T_delta), zeros(3,6)
    zeros(3,3), R_U_0*1, R_star
    L_3DOF*(M_tot\B_delta), -L_3DOF*Lg*g, -L_3DOF*(M_tot\D_star)-L_3DOF*Lc*U_0];

G.B = [inv(T_delta);zeros(6,3)];

G.C = [zeros(3),eye(3),zeros(3)
    zeros(3,6),eye(3)];

G.n = size(G.A,1);
NMEAS = size(G.C,1);
NCON = size(G.B,2);

G.D = zeros(NMEAS,NCON);

s = zpk('s');

%% State-space representation of weighting transfer matrices
% Disturbance (Wd):
    % Dimension
    Wd.n = NCON;

    % L
    Wd.L = [zeros(G.n-Wd.n,Wd.n);L_3DOF*(M_tot\B_delta)*100000]; % 100000

    % Band-pass min and max frequencies 
    Wd.omega_L = 0.2; % 0.2
    Wd.omega_H = 2; % 2
    
    % Gain of band-pass filter
    Wd.c = 1;  

    % A_d1
    Wd.A_d1 = -Wd.omega_H*diag([1 1 1]);

    % B_d1
    Wd.B_d1 = -Wd.omega_L*Wd.c*diag([1 1 1]);
    
    % Omega_d2
    Wd.Omega_d2 = -(Wd.omega_L * Wd.omega_H)*diag([1 1 1]);
    
    % Lambda_d2
    Wd.Lambda_d2 = -(Wd.omega_L + Wd.omega_H)*diag([1 1 1]);
    
    % A_d2
    Wd.A_d2 = [zeros(Wd.n),eye(Wd.n); Wd.Omega_d2, Wd.Lambda_d2];
    
    % B_d2
    Wd.B_d2 = -Wd.c*Wd.omega_L^2*diag([1 1 1]);
    
    % Transfer function for analysis
    Wd.tf = Wd.c*Wd.omega_H*s/((s+Wd.omega_L)*(s+Wd.omega_H));

 % Sensor noise (Wv)
    % Dimension
    Wv.n = NMEAS;
    
    % Gain
    Wv.c = 1e3;% 1000

    % High-pass frequency
    Wv.omega_H = 1000;
    Wv.omega_H = 10; % 10
    
    % A_w2_LP 
    Wv.A_w2_LP = -Wv.omega_H*eye(Wv.n);
    
    % B_w2 
    Wv.B_w2_LP = Wv.omega_H*eye(Wv.n);
    
    % Transfer function for analysis
    Wv.tf = 1 - Wv.omega_H/(s+Wv.omega_H);
    
% Output (Wy):
    % Dimension
    Wy.n = NMEAS;
    
    % Gain
    Ap = 10;

    % Low-pass frequency
    Wy.omega_L = 1; % 1
    
    % A_z1 
    Wy.A_z1 = -Wy.omega_L*eye(Wy.n);
    
    % B_z1 
    Wy.B_z1 = Ap*Wy.omega_L*eye(Wy.n);
    
    % Transfer function for analysis
    Wy.tf = Wy.omega_L/(s+Wy.omega_L);
    
% Controller effort (Wu)
    % Dimension
    Wu.n = NCON;
    
    % Gain
    Wu.c = 100;
    Wu.c = 1; % 1
    
    % High-pass frequency   
    Wu.omega_H_a = 2; % 2
    Wu.omega_H_b = 20; % 0.2
    
    
    % A_z2,dyn 
    Wu.A_z2_dyn = -Wu.omega_H_b*diag([1 1 1]);
    
    % B_z1 
    Wu.B_z2_dyn = (Wu.omega_H_a - Wu.omega_H_b)*diag([1 1 1]);
    
    % Transfer function for analysis
    Wu.tf = (s+Wu.omega_H_a)/(s+Wu.omega_H_b);
    
    
%% Complete plant
clear P;
P.A = [G.A, Wd.L, Wd.L, zeros(G.n,Wd.n+Wy.n+Wu.n)
    zeros(Wd.n,G.n), Wd.A_d1, zeros(Wd.n,2*Wd.n+Wy.n+Wu.n)
    zeros(Wd.n,G.n+2*Wd.n), eye(Wd.n), zeros(Wd.n,Wy.n+Wu.n)
    zeros(Wd.n,G.n+Wd.n), Wd.Omega_d2, Wd.Lambda_d2, zeros(Wd.n,Wy.n+Wu.n)
    -Wy.B_z1*G.C, zeros(Wy.n,3*Wd.n), Wy.A_z1, zeros(Wy.n,Wu.n)
    zeros(Wu.n,G.n+3*Wd.n+Wy.n), Wu.A_z2_dyn];

P.B = [zeros(G.n,Wd.n+Wy.n), G.B
    zeros(Wd.n,Wy.n), Wd.B_d1, zeros(Wd.n,Wu.n)
    zeros(Wd.n,Wy.n+Wd.n+Wu.n)
    zeros(Wd.n,Wy.n), Wd.B_d2, zeros(Wd.n,Wu.n)
    Wy.B_z1, zeros(Wy.n,Wd.n), zeros(Wy.n,Wu.n)
    zeros(Wu.n,Wy.n+Wd.n), Wu.B_z2_dyn];

P.C = [zeros(Wy.n,G.n+3*Wd.n), eye(Wy.n), zeros(Wy.n,Wu.n)
    zeros(Wu.n,G.n+3*Wd.n+Wy.n), -eye(Wu.n)*Wu.c
    -G.C, zeros(Wy.n,3*Wd.n+Wy.n+Wu.n)];
   
P.D = [zeros(Wy.n,Wy.n+Wd.n+Wu.n)
    zeros(Wu.n,Wy.n+Wd.n), eye(Wu.n)*Wu.c
    eye(Wy.n), zeros(Wy.n,Wd.n), -G.D];

%%

% Including sensor noise (can be commented out)

P.B = [P.B(:,1:Wd.n+Wy.n), zeros(size(P.B,1),Wv.n), P.B(:,size(P.B,2)-Wu.n+1:size(P.B,2))];
P.D = [P.D(:,1:Wd.n+Wy.n), [zeros(Wy.n+Wu.n,Wv.n); -Wv.c*eye(Wv.n)], P.D(:,Wd.n+Wy.n+1:size(P.D,2))];

P = ss(P.A,P.B,P.C,P.D);

%% Controller
[H2.K,H2.CL,H2.GAMMA,H2.INFO] = h2syn(P,NMEAS,NCON); 

% Initial condition
H2.x_0 = zeros(size(H2.K.A,1),1);


% Reference, if constant
H2.u_1 = [z_d,0,0,0,0,0]';


%%

Kalman.X_0 = zeros(12,1);
Kalman.X_0(7) = z_d;
