% Rectangular prism class

classdef RectangularCuboid < PhysicalObject
    properties
        length,     % Size in x-direction (length)
        width;      % Size in y-direction (width)
        height;     % Size in z-direction (height)
%         phi;    % Rotation about x-axis
%         theta;  % Rotation about y-axis
%         psi;    % Rotation about z-axis
    end

    methods 
        function obj = RectangularCuboid(mass,length,width,height)
            I_x = mass*(width^2 + height^2)/12;
            I_y = mass*(length^2 + height^2)/12;
            I_z = mass*(width^2 + length^2)/12;
            I = [I_x 0 0; 0 I_y 0; 0 0 I_z];
            
            obj@PhysicalObject(mass,I);    % Superclass constructor
            
            obj.length = length;
            obj.width = width;
            obj.height = height;
        end
        function obj = plotXY(obj,x,y,R)
%             obj.plotAB(obj.length,obj.width,x,y);
            p = corners(obj);
            p_00 = R*p{1,1,1};
            p_01 = R*p{1,2,1};
            p_10 = R*p{2,1,1};
            p_11 = R*p{2,2,1};
            obj.plot12(x,y,p_00(1:2),p_01(1:2),p_10(1:2),p_11(1:2));
        end
        function obj = plotXZ(obj,x,z,R)
%             obj.plotAB(obj.length,obj.height,x,z);
            p = corners(obj);
            p_00 = R*p{1,1,1};
            p_01 = R*p{1,1,2};
            p_10 = R*p{2,1,1};
            p_11 = R*p{2,1,2};
            obj.plot12(x,z,p_00([1,3]),p_01([1,3]),p_10([1,3]),p_11([1,3]));
        end
        function obj = plotYZ(obj,y,z,R)
%             obj.plotAB(obj.width,obj.height,y,z);
            p = corners(obj);
            p_00 = R*p{1,1,1};
            p_01 = R*p{1,1,2};
            p_10 = R*p{1,2,1};
            p_11 = R*p{1,2,2};
            obj.plot12(y,z,p_00(2:3),p_01(2:3),p_10(2:3),p_11(2:3));
        end
        
        function obj = plotAB(obj,length,height,x,y)
            hold on
            plot([x-length/2,x+length/2],[y+height/2,y+height/2])
            hold on
            plot([x-length/2,x+length/2],[y-height/2,y-height/2])
            hold on
            plot([x-length/2,x-length/2],[y-height/2,y+height/2])
            hold on
            plot([x+length/2,x+length/2],[y-height/2,y+height/2])
            hold off
            axis normal
            grid on
        end
        %%
        function p = corners(obj)
           p{1,1,1} = [-obj.length/2,-obj.width/2,-obj.height/2]';
           p{1,1,2} = [-obj.length/2,-obj.width/2,obj.height/2]';
           p{1,2,1} = [-obj.length/2,obj.width/2,-obj.height/2]'; 
           p{1,2,2} = [-obj.length/2,obj.width/2,obj.height/2]';
           p{2,1,1} = [obj.length/2,-obj.width/2,-obj.height/2]';
           p{2,1,2} = [obj.length/2,-obj.width/2,obj.height/2]';
           p{2,2,1} = [obj.length/2,obj.width/2,-obj.height/2]'; 
           p{2,2,2} = [obj.length/2,obj.width/2,obj.height/2]';
        end
            
            
        function obj = plot12(obj,x,y,p_00,p_01,p_10,p_11)
            hold on
            plot([x + p_00(1), x + p_10(1)],[y + p_00(2), y + p_10(2)])
            hold on
            plot([x + p_01(1), x + p_11(1)],[y + p_01(2), y + p_11(2)])
            hold on
            plot([x + p_00(1), x + p_01(1)],[y + p_00(2), y + p_01(2)])
            hold on
            plot([x + p_10(1), x + p_11(1)],[y + p_10(2), y + p_11(2)])
            hold off
            axis normal
            grid on
        end
        %%
    end
    
end