function waveData = getWaveDataset( U_wind, theta_0, N, K, phaseOn)
% Generates a linear wave model with N frequency components in K
% directions. Main propagation direction in theta_0. PM spectrum for wind
% speed U_wind (19.4 m above surface) is used
% phaseOn is boolean

if (phaseOn ~= 0 && phaseOn ~= 1)
    return;
end

theta_max = 65*pi/180;
g = 9.82;
% d_theta = pi/K;
d_theta = 2*theta_max/K;

waveData = zeros(N*K,5);

% Frequebcy model:
A =8.1e-3 * g^2;
B = 0.74*(g/U_wind)^4;

% Find frequency limits and dw
w = 0.01;
while PM_freq_specter(w,U_wind) < 0.05
    w = w + 0.01;
end
w_L = w;
while PM_freq_specter(w,U_wind) > 0.05
    w = w + 0.01;
end
w_H = w;
dw = (w_H-w_L)/N;
w = w_L + dw/2;

for n = 1:N
    S = PM_freq_specter(w,U_wind);
    
    theta = theta_0 - theta_max + d_theta/2;
    for k = 1:K
        f = (2/pi)*cos(theta-theta_0)^2;
       
        zeta_a = sqrt(2*S*f*dw*d_theta);
        waveData((n-1)*K+k,1) = zeta_a;
        waveData((n-1)*K+k,2) = w;
        waveData((n-1)*K+k,3) = phaseOn*2*pi*rand;
        waveData((n-1)*K+k,4) = theta;
        waveData((n-1)*K+k,5) = S*f;

        theta = theta + d_theta;
    end
    w = w + dw;
    
end
 
end

